-------------------------------------------------------------------------------
--  Copyright (c) 2018 Eclektek LLC. All rights reserved.
--
--
--  Developed by: William Carter
--
--  Permission is hereby granted, free of charge, to any person obtaining a copy of this software and  
--  associated documentation files (the "Software"), to deal with the Software without restriction,  
--  including without limitation the rights to use, copy, modify, merge, publish, sublicense and/or    
--  distribute copies of the Software, and to permit persons to whom the Software is furnished to do so,  
--  subject to the following conditions:
--
--      Redistributions of source code must retain the above copyright notice, this list of conditions 
--      and the following disclaimers.
--      Redistributions in binary form must reproduce the above copyright notice, this list of conditions 
--      and the following disclaimers in the documentation and/or other materials provided with the distribution.
--      Neither the names of William Carter, Eclektek LLC, nor the names of its 
--      contributors may be used to endorse or promote products derived from this Software without 
--      specific prior written permission from Eclektek LLC.
--      This Software is provided free of charge and Eclektek LLC reserves the right to sell copies of the 
--      Software in it original form. Anyone sub-licensing this Software will provide it free of charge.
--
--  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT 
--  LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. 
--  IN NO EVENT SHALL THE CONTRIBUTORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, 
--  WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE 
--  SOFTWARE OR THE USE OR OTHER DEALINGS WITH THE SOFTWARE.
-------------------------------------------------------------------------------
--  History :                                                                       
--                                                                                                                  
--  Date      By    Comments                                         
--  --------  ----  ------------------------------------------------------------
--  10/16/18  W.C.  Created
--
-------------------------------------------------------------------------------
--  Project       : ETHERNET_TESTER_PUBLIC                                                          
--  PCB           : Nexys Video
--  Device        : XC7A200T-1SBG484C
--  Toolchain     : Vivado 2016.2
--  Entity Name   : NexysVideo_DUT
--  File Name     : NexysVideo_DUT.vhd
--  Design Parent : none                                                          
--                                                                             
-------------------------------------------------------------------------------
--  Dependencies:
--
--  NexysVideo_DUT.vhd is uses the following components:
--    none                 none
--
--  NexysVideo_DUT.vhd is uses the following packages:
--    Package        none
--
-------------------------------------------------------------------------------
--  NexysVideo_DUT.vhd
--
--   
--  
-------------------------------------------------------------------------------
--  Limitations
--
------------------------------------------------------------------------------
--  ToDo
--
-------------------------------------------------------------------------------
--  NOTES:
--
--
--
--
--
-------------------------------------------------------------------------------
--  CAUTIONS!!!
--  
-------------------------------------------------------------------------------

Library ieee;
  use ieee.std_logic_1164.all;
  use ieee.std_logic_arith.all;
  use ieee.std_logic_unsigned.all;

library UNISIM;
  use UNISIM.vcomponents.all;
    
entity NexysVideo_DUT is
  port(
        -- GLOBAL SIGNALS
        sysclk                          : in    std_logic; -- global external 100 MHz clock        
        cpu_resetn                      : in    std_logic; -- normally '1', '0' when Reset button pressed 
        
        -- General I/O resources
        btnc                            : in    std_logic; -- Vadj HR Bank 16 normally '0', '1' when pressed
        btnd                            : in    std_logic; -- Vadj HR Bank 16 normally '0', '1' when pressed
        btnl                            : in    std_logic; -- Vadj HR Bank 16 normally '0', '1' when pressed
        btnr                            : in    std_logic; -- Vadj HR Bank 16 normally '0', '1' when pressed
        btnu                            : in    std_logic; -- Vadj HR Bank 16 normally '0', '1' when pressed
        sw                              : in    std_logic_vector(7 downto 0); -- Vadj HR Bank 15 or 16
        led                             : out   std_logic_vector(7 downto 0); -- VCC2V5 HR bank 13
       
        
        -- PHY interface
        eth_int_n                       : in    std_logic;                          
        eth_mdc                         : out   std_logic;                          
        eth_mdio                        : inout std_logic;                          
        eth_reset_n                     : out   std_logic;                          
        eth_rxclk                       : in    std_logic;                          
        eth_rxctl                       : in    std_logic;                          
        eth_rxd                         : in    std_logic_vector(3 downto 0);       
        eth_txclk                       : out   std_logic;                          
        eth_txctl                       : out   std_logic;                          
        eth_txd                         : out   std_logic_vector(3 downto 0);        
                                                                                       
        -- UART                                                                     
        uart_rx_out                     : out   std_logic;                          
        uart_tx_in                      : in    std_logic;                          
                                                                                    
        -- PMOD JC                                                                  
        JA                              : inout std_logic_vector(7 downto 0)         
  );
end entity;

architecture rtl of NexysVideo_DUT is

-------------------------------------------------------------------------------
--  Component Declarations
-------------------------------------------------------------------------------
  component ETH_DUT
  port(
        -- GLOBAL SIGNALS
        clk100                : in    std_logic; -- 100 MHz clock for controls
        clk125                : in    std_logic; -- 125 MHz clock for Ethernet transmitter clock domain
        rxclk                 : in    std_logic; -- 125 MHz clock for Ethernet receiver clock domain 
        areset                : in    std_logic; -- external asynchronous reset
        
        phyrxer               : in    std_logic;
        phy_rxdv              : in    std_logic;
        phy_rxd               : in    std_logic_vector(7 downto 0);
        phy_txen              : out   std_logic;
        phy_txd               : out   std_logic_vector(7 downto 0);
        mdclk_out             : out   std_logic;
        mdio_in               : in    std_logic;
        mdio_trst_en          : out   std_logic;
                        
        -- TTL USB UART
        UartRx                : in    std_logic; -- Adafruit TTL UART Green wire
        UartTx                : out   std_logic   -- Adafruit TTL UART White wire 
  );
  end component;

  component clk_wiz_0
  port
   (-- Clock in ports
    clk_in1           : in     std_logic;
    -- Clock out ports
    clk_out1          : out    std_logic;
    clk_out2          : out    std_logic;
    clk_out3          : out    std_logic;
    -- Status and control signals
    reset             : in     std_logic;
    locked            : out    std_logic
   );
  end component;

-------------------------------------------------------------------------------
--  Type Declarations
-------------------------------------------------------------------------------


-------------------------------------------------------------------------------
--  Constant Declarations
-------------------------------------------------------------------------------

  constant CLOCK_125MHZ_10mS_RESET_COUNT        : std_logic_vector(23 downto 0) := X"1312D0";   -- 10ms reset for hardware
  constant CLOCK_125MHZ_1us_RESET_COUNT         : std_logic_vector(23 downto 0) := X"00007D";   -- 1us reset for simulation

-------------------------------------------------------------------------------
--  Signal Declarations
-------------------------------------------------------------------------------


signal areset               : std_logic;
signal clk_100              : std_logic;
signal clk_125              : std_logic;
signal clk_125ps90          : std_logic;
signal phy_gtx_clk          : std_logic;
signal tx_gtxclk            : std_logic;
signal dcm_locked           : std_logic;
signal ts                   : std_logic_vector(47 downto 0);
signal rst_del, rst_n       : std_logic;

signal rxerr_led_indicator  : std_logic;
signal gmii_rxdv            : std_logic;
signal gmii_rxd             : std_logic_vector(7 downto 0);
signal gmii_txen            : std_logic;                   
signal gmii_txd             : std_logic_vector(7 downto 0);
signal dbg_txen             : std_logic;
signal dbg_rxdv             : std_logic;
signal db_reset             : std_logic;   
signal arst_count           : std_logic_vector(23 downto 0);  
signal arst_loadcount       : std_logic_vector(23 downto 0);
signal is_simulation        : std_logic;
 
begin
                                                  

-------------------------------------------------------------------------------
--  Component Instantiations
-------------------------------------------------------------------------------

   MMCM_INST : clk_wiz_0
   port map ( 
     clk_in1    => sysclk,
     clk_out1   => clk_100,
     clk_out2   => clk_125,
     clk_out3   => clk_125ps90,
     reset      => '0',
     locked     => dcm_locked            
 );
  
  
  
  -- Ethernet PHY  4-bit DDR data
  -- ODDR: Output Double Data Rate Output Register with Set, Reset, and Clock Enable.  
  -- Xilinx 7 
  -- Series HDL Libraries Guide, version 13.4
  GEN_oddr : for i in 3 downto 0 generate
    ODDR_inst : ODDR
      generic map(
      DDR_CLK_EDGE  => "SAME_EDGE", -- "OPPOSITE_EDGE" or "SAME_EDGE"
      INIT          => '0', -- Initial value for Q port ('1' or '0')
      SRTYPE        => "SYNC") -- Reset Type ("ASYNC" or "SYNC")
    port map (
      Q     => eth_txd(i), -- 1-bit DDR output
      C     => clk_125, -- 1-bit clock input
      CE    => '1', -- 1-bit clock enable input
      D1    => gmii_txd(i),   -- 1-bit data input (positive edge)
      D2    => gmii_txd(i+4), -- 1-bit data input (negative edge)
      R     => '0', -- 1-bit reset input
      S     => '0' -- 1-bit set input
    );  
  end generate;
  
  -- Ethernet PHY  Tx Enable
  -- ODDR: Output Double Data Rate Output Register with Set, Reset, and Clock Enable.  
  -- Xilinx 7 
  -- Series HDL Libraries Guide, version 13.4
    ODDR_CTRL_inst : ODDR
      generic map(
      DDR_CLK_EDGE  => "SAME_EDGE", -- "OPPOSITE_EDGE" or "SAME_EDGE"
      INIT          => '0', -- Initial value for Q port ('1' or '0')
      SRTYPE        => "SYNC") -- Reset Type ("ASYNC" or "SYNC")
    port map (
      Q     => eth_txctl, -- 1-bit DDR output
      C     => clk_125, -- 1-bit clock input
      CE    => '1', -- 1-bit clock enable input
      D1    => gmii_txen,   -- 1-bit data input (positive edge)
      D2    => gmii_txen, -- 1-bit data input (negative edge)
      R     => '0', -- 1-bit reset input
      S     => '0' -- 1-bit set input
    );  
  
  -- Ethernet PHY Tx clock
  -- ODDR: Output Double Data Rate Output Register with Set, Reset, and Clock Enable.  
  -- Xilinx 7 
  -- Series HDL Libraries Guide, version 13.4
    ODDR_CLK_inst : ODDR
      generic map(
      DDR_CLK_EDGE  => "SAME_EDGE", -- "OPPOSITE_EDGE" or "SAME_EDGE"
      INIT          => '0', -- Initial value for Q port ('1' or '0')
      SRTYPE        => "SYNC") -- Reset Type ("ASYNC" or "SYNC")
    port map (
      Q     => eth_txclk, -- 1-bit DDR output
      C     => clk_125ps90, -- 1-bit clock input
      CE    => '1', -- 1-bit clock enable input
      D1    => '1',   -- 1-bit data input (positive edge)
      D2    => '0', -- 1-bit data input (negative edge)
      R     => '0', -- 1-bit reset input
      S     => '0' -- 1-bit set input
    );  

  -- RGMII Rx DDR data  
  GEN_iddr_d : for i in 3 downto 0 generate
    -- IDDR: Double Data Rate Input Register with Set, Reset
    -- and Clock Enable.
    -- 7 Series
    -- Xilinx HDL Libraries Guide, version 13.4
    IDDR_inst_data : IDDR
    generic map (
      DDR_CLK_EDGE    => "SAME_EDGE_PIPELINED", -- "OPPOSITE_EDGE", "SAME_EDGE"  or "SAME_EDGE_PIPELINED"
      INIT_Q1         => '0', -- Initial value of Q1: '0' or '1'
      INIT_Q2         => '0', -- Initial value of Q2: '0' or '1'
      SRTYPE          => "SYNC") -- Set/Reset type: "SYNC" or "ASYNC"
    port map (
      Q1  => gmii_rxd(i), -- 1-bit output for positive edge of clock
      Q2  => gmii_rxd(i+4), -- 1-bit output for negative edge of clock
      C   => eth_rxclk, -- 1-bit clock input
      CE  => '1', -- 1-bit clock enable input
      D   => eth_rxd(i), -- 1-bit DDR data input
      R   => '0', -- 1-bit reset
      S   => '0' -- 1-bit set
    );    
  end generate;

  -- RGMII DDR Rx data valid data  
  -- IDDR: Double Data Rate Input Register with Set, Reset
  -- and Clock Enable.
  -- 7 Series
  -- Xilinx HDL Libraries Guide, version 13.4
  IDDR_inst : IDDR
  generic map (
    DDR_CLK_EDGE    => "SAME_EDGE_PIPELINED", -- "OPPOSITE_EDGE", "SAME_EDGE"  or "SAME_EDGE_PIPELINED"
    INIT_Q1         => '0', -- Initial value of Q1: '0' or '1'
    INIT_Q2         => '0', -- Initial value of Q2: '0' or '1'
    SRTYPE          => "SYNC") -- Set/Reset type: "SYNC" or "ASYNC"
  port map (
    Q1  => gmii_rxdv, -- 1-bit output for positive edge of clock
    Q2  => open, -- 1-bit output for negative edge of clock
    C   => eth_rxclk, -- 1-bit clock input
    CE  => '1', -- 1-bit clock enable input
    D   => eth_rxctl, -- 1-bit DDR data input
    R   => '0', -- 1-bit reset
    S   => '0' -- 1-bit set
  );    
    
   
  DUT_inst : ETH_DUT
  port map(
        clk100       => clk_100,                         
        clk125       => clk_125,                   
        rxclk        => eth_rxclk,                                 
        areset       => areset,                                
        phyrxer      => '0',            -- RGMII interface doesn't support phy_rxer                  
        phy_rxdv     => gmii_rxdv,               
        phy_rxd      => gmii_rxd,                      
        phy_txen     => gmii_txen,                       
        phy_txd      => gmii_txd,                        
        mdclk_out    => open,           -- not implemented
        mdio_in      => '1',            -- not implemented
        mdio_trst_en => open,           -- not implemented
        UartRx       => uart_tx_in,
        UartTx       => uart_rx_out);
  
          
                               
-------------------------------------------------------------------------------
--  Concurrent Assignments
-------------------------------------------------------------------------------

  areset  <= not cpu_resetn; 

  dbg_txen  <= '1' when gmii_txen = '1' else '0';
  dbg_rxdv  <= '1' when gmii_rxdv = '1'  else '0';
                                                                    
-------------------------------------------------------------------------------
--  Process Declarations
-------------------------------------------------------------------------------


  -- timestamp fre-running counter
  process(clk_100,areset)
  begin
    if areset = '1' then
      ts <= (others => '0');
    elsif rising_edge(clk_100) then
      ts <= ts + 1;
    end if;       
  end process;
  
  -- control the button reset for the PHY
  
  arst_loadcount  <= CLOCK_125MHZ_10mS_RESET_COUNT when is_simulation = '0' else
                     CLOCK_125MHZ_1us_RESET_COUNT;
                      
  db_reset <= '1' when arst_count < arst_loadcount else '0';

  process (clk_125, areset)
  begin  -- process
    if rising_edge(clk_125)  then  -- rising clock edge
      if areset = '1'  then
        arst_count   <= (others => '0');
      elsif arst_count < arst_loadcount then  
        arst_count   <= arst_count + 1;
      else  
        arst_count   <= arst_count;
      end if;
    end if;
  end process; 
  
      
-------------------------------------------------------------------------------
--  Other Output Assignments
-------------------------------------------------------------------------------
     
  Led(7)  <= ts(26);
  Led(6)  <= sw(7);
  Led(5)  <= rxerr_led_indicator;
  Led(4 downto 4) <= (others => 'Z');
  Led(3 downto 0) <= sw(3 downto 0);
  -- output assignments

  
  JA(7)   <=  'Z';            
  JA(6)   <=  'Z';            
  JA(5)   <=  'Z';                
  JA(4)   <=  'Z';            
  JA(3)   <=  'Z';  
  JA(2)   <=  'Z';         
  JA(1)   <=  dbg_rxdv;                  
  JA(0)   <=  dbg_txen;                  
                                         
  eth_mdio    <= 'Z';                   
  eth_mdc     <= '0';                    
--  eth_txctl   <= i_txen;                
--  eth_txd     <= i_txd;                 
--  eth_reset_n <= rst_n;
--  phy_txer    <= '0';                             
                                                    
end rtl;
